import { __decorate } from "tslib";
import { Combobox as FASTComboBox } from '@microsoft/fast-components';
import { DOM, Observable, observable } from '@microsoft/fast-element';
import { foundationComboboxStyles as styles } from './combobox.styles';
import { foundationComboboxTemplate as template } from './combobox.template';
export const wasClickOutsideElement = (event, target) => !event.composedPath().includes(target);
export class Combobox extends FASTComboBox {
    get isListboxVisible() {
        var _a;
        return !((_a = this.shadowRoot.querySelector('[part="listbox"]')) === null || _a === void 0 ? void 0 : _a.hasAttribute('hidden'));
    }
    constructor() {
        super();
        this.open = false;
        this.inputSelector = 'input';
        this.indicatorSelector = '.combo-box-indicator';
        this.addEventListener('focus', this.handleFocus);
        this.addEventListener('keydown', this.handleInputKeyDown);
        this.addEventListener('click', this.handleComboboxClick);
    }
    focusInputElement() {
        DOM.queueUpdate(() => {
            var _a;
            const inputElement = (_a = this.shadowRoot) === null || _a === void 0 ? void 0 : _a.querySelector(this.inputSelector);
            inputElement.focus();
        });
    }
    handleComboboxClick(event) {
        DOM.queueUpdate(() => {
            this.focusInputElement();
        });
    }
    handleFocus(event) {
        this.focusInputElement();
    }
    handleInputKeyDown(event) {
        if (event.key.length === 1 && !this.isListboxVisible) {
            this.open = true;
            this.focusInputElement();
        }
    }
    handleClickArrow(event) {
        DOM.queueUpdate(() => {
            if (this.isListboxVisible) {
                this.open = false;
            }
            else {
                this.open = true;
            }
            event.preventDefault();
        });
        event.preventDefault();
    }
    updateIndicatorIcon() {
        var _a;
        const indicatorElement = (_a = this.shadowRoot) === null || _a === void 0 ? void 0 : _a.querySelector(this.indicatorSelector);
        if (indicatorElement) {
            if (this.hasAttribute('open')) {
                indicatorElement.classList.add('open'); // Add the open class
            }
            else {
                indicatorElement.classList.remove('open'); // Remove the open class
            }
        }
    }
    attributeChangedCallback(name, oldValue, newValue) {
        super.attributeChangedCallback(name, oldValue, newValue);
        if (name === 'open') {
            // Update the indicator icon when the 'open' attribute changes
            this.updateIndicatorIcon();
        }
    }
    connectedCallback() {
        var _a;
        super.connectedCallback();
        const arrowElement = (_a = this.shadowRoot) === null || _a === void 0 ? void 0 : _a.querySelector(this.indicatorSelector);
        if (arrowElement) {
            arrowElement.addEventListener('click', this.handleClickArrow.bind(this));
        }
    }
    findCaseInsensitiveIndex(text) {
        if (this.options) {
            return this.options.findIndex((el) => el.text === text);
        }
        return -1;
    }
    get value() {
        Observable.track(this, 'value');
        return this['_value'];
    }
    set value(next) {
        var _a, _b, _c;
        const prev = `${this['_value']}`;
        if (this.$fastController.isConnected && this.options) {
            const selectedIndex = this.findCaseInsensitiveIndex(next);
            const prevSelectedValue = (_a = this.options[this.selectedIndex]) === null || _a === void 0 ? void 0 : _a.text;
            const nextSelectedValue = (_b = this.options[selectedIndex]) === null || _b === void 0 ? void 0 : _b.text;
            this.selectedIndex =
                prevSelectedValue !== nextSelectedValue ? selectedIndex : this.selectedIndex;
            next = ((_c = this.firstSelectedOption) === null || _c === void 0 ? void 0 : _c.text) || next;
        }
        if (prev !== next) {
            this['_value'] = next;
            super.valueChanged(prev, next);
            Observable.notify(this, 'value');
        }
    }
}
__decorate([
    observable
], Combobox.prototype, "open", void 0);
// explicitly declaring for extensibility
export const foundationComboboxShadowOptions = undefined;
export const defaultComboboxConfig = {
    indicator: `
        <svg
            class="combo-box-indicator"
            part="combo-box-indicator"
            viewBox="0 0 12 7"
            xmlns="http://www.w3.org/2000/svg"
        >
            <path
                d="M11.85.65c.2.2.2.5 0 .7L6.4 6.84a.55.55 0 01-.78 0L.14 1.35a.5.5 0 11.71-.7L6 5.8 11.15.65c.2-.2.5-.2.7 0z"
            />
        </svg>
    `,
};
/**
 * The Foundation Combo-box
 *
 * @public
 * @remarks
 * HTML Element: \<foundation-combo-box\>
 */
export const foundationCombobox = Combobox.compose(Object.assign({ baseName: 'combobox', template,
    styles, shadowOptions: foundationComboboxShadowOptions }, defaultComboboxConfig));
