import { __awaiter, __decorate } from "tslib";
import { Datasource } from '@genesislcap/foundation-comms';
import { NotificationEvent, showNotification, getNotificationBuilder, getToastBuilder, } from '@genesislcap/foundation-notifications';
import { createLogger } from '@genesislcap/foundation-utils';
import { attr, nullableNumberConverter, observable } from '@microsoft/fast-element';
import { FoundationElement } from '@microsoft/fast-foundation';
import { foundationNotificationListenerTemplate as template } from './notification-listener.template';
export const foundationNotificationListenerShadowOptions = {
    mode: 'open',
    delegatesFocus: true,
};
/**
 * @tagname %%prefix%%-notification-listener
 */
export class NotificationListener extends FoundationElement {
    constructor() {
        super(...arguments);
        this.logger = createLogger('notification-listener');
        /**
         * Number of milliseconds before toast will disappear. Leave empty or set to 0 to keep toast open indefinitely
         * @public
         */
        this.closeTimeout = 0;
        /**
         * Array of buttons for notification toasts
         * @public
         */
        this.toastButtons = [];
    }
    connectedCallback() {
        const _super = Object.create(null, {
            connectedCallback: { get: () => super.connectedCallback }
        });
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            _super.connectedCallback.call(this);
            this.addEventListener(NotificationEvent.NOTIFICATION_EVENT, (e) => {
                e.stopImmediatePropagation();
                showNotification(e.detail, this.tagName, this.notificationListener);
            });
            if (this.resourceName) {
                yield this.datasource.init({
                    resourceName: this.resourceName,
                }, true);
                (_a = this.datasource.stream) === null || _a === void 0 ? void 0 : _a.subscribe((result) => {
                    var _a, _b;
                    if (!result.ROW) {
                        return;
                    }
                    const details = result === null || result === void 0 ? void 0 : result.ROW[0];
                    if (result.SEQUENCE_ID > 1 && details.DETAILS.OPERATION === 'INSERT') {
                        const title = details.HEADER;
                        const message = details.MESSAGE;
                        const type = (_a = details.NOTIFY_SEVERITY) === null || _a === void 0 ? void 0 : _a.toLowerCase();
                        const createdAt = details.CREATED_AT ? new Date(details.CREATED_AT) : undefined;
                        const expiry = details.EXPIRY ? details.EXPIRY - details.CREATED_AT : 0;
                        let buttons = [];
                        if (this.toastButtons && this.toastButtons.length > 0) {
                            buttons = this.toastButtons.filter(({ condition }) => !!condition ? condition({ details }) : true);
                        }
                        showNotification(getNotificationBuilder()
                            .withTitle(title)
                            .withBody(message)
                            .withCreatedAt(createdAt)
                            .withToast(getToastBuilder()
                            .withDetails(details)
                            .withType(type)
                            .withButtons(buttons)
                            .withAutoClose((_b = !!expiry) !== null && _b !== void 0 ? _b : !!this.closeTimeout)
                            .withCloseTimeout(expiry !== null && expiry !== void 0 ? expiry : this.closeTimeout)
                            .build())
                            .build(), this.tagName, this.notificationListener);
                    }
                });
            }
        });
    }
}
__decorate([
    observable
], NotificationListener.prototype, "notificationListener", void 0);
__decorate([
    attr({ attribute: 'resource-name' })
], NotificationListener.prototype, "resourceName", void 0);
__decorate([
    attr({ attribute: 'close-timeout', converter: nullableNumberConverter })
], NotificationListener.prototype, "closeTimeout", void 0);
__decorate([
    observable
], NotificationListener.prototype, "toastButtons", void 0);
__decorate([
    Datasource
], NotificationListener.prototype, "datasource", void 0);
/**
 * The Foundation Notification Listener
 *
 * @public
 * @remarks
 * HTML Element: \<foundation-notification-listener\>
 */
export const foundationNotificationListener = NotificationListener.compose({
    baseName: 'notification-listener',
    template,
    shadowOptions: foundationNotificationListenerShadowOptions,
});
